Package: OpenMx
Date: 2018-03-16
Title: Extended Structural Equation Modelling
Authors@R: c(person(c("Steven", "M."), "Boker", role="aut"),
    person(c("Michael", "C."), "Neale", role="aut"), person(c("Hermine", "H."),
    "Maes", role="aut"), person(c("Michael", "J."), "Wilde", role="ctb"), person("Michael",
    "Spiegel", role="aut"), person(c("Timothy", "R."), "Brick", role="aut"),
    person("Ryne", "Estabrook", role="aut"), person(c("Timothy", "C."),
    "Bates", role="aut"), person("Paras", "Mehta", role="ctb"), person("Timo", c("von",
    "Oertzen"), role="ctb"), person(c("Ross", "J."), "Gore", role="aut"),
    person(c("Michael", "D."), "Hunter", role="aut"), person(c("Daniel", "C."), "Hackett", role="ctb"),
    person("Julian", "Karch", role="ctb"), person(c("Andreas", "M."), "Brandmaier", role="ctb"),
    person(c("Joshua", "N."), "Pritikin", email="jpritikin@pobox.com",
    role=c("aut", 'cre')), person("Mahsa", "Zahery", role="aut"),
    person(c("Robert", "M."), "Kirkpatrick", role="aut"),
    person(c("Yang"), "Wang", role="ctb"),
    person(c("Charles"), "Driver", email="driver@mpib-berlin.mpg.de", role="ctb"),
    person("Massachusetts Institute of Technology", role="cph"),
    person("S. G.", "Johnson", role="cph"),
    person("Association for Computing Machinery", role="cph"),
    person("Dieter", "Kraft", role="cph"),
    person("Stefan", "Wilhelm", role="cph"),
    person("Manjunath B G", role="cph"))
URL: http://openmx.ssri.psu.edu, https://github.com/OpenMx/OpenMx
BugReports: http://openmx.ssri.psu.edu/forums
Description: Facilitates treatment of statistical model specifications
    as things that can be generated and manipulated programmatically.
    Structural equation models may be specified with reticular action model matrices or paths,
    linear structural relations matrices or paths, or
    directly in matrix algebra.
    Fit functions include full information maximum likelihood,
    maximum likelihood, and weighted least squares.
    Example models include confirmatory factor, multiple group, mixture
    distribution, categorical threshold, modern test theory, differential
    equations, state space, and many others.  MacOS users can download the most up-to-date 
    package binaries from <http://openmx.ssri.psu.edu>.
    See Neale, Hunter, Pritikin, Zahery, Brick,
    Kirkpatrick, Estabrook, Bates, Maes, & Boker (2016) <doi:10.1007/s11336-014-9435-8>.
SystemRequirements: GNU make
ByteCompile: yes
License: Apache License (== 2.0)
LinkingTo: Rcpp, RcppEigen, StanHeaders (>= 2.10.0.2), BH (>= 1.60),
        rpf (>= 0.45)
Imports: Rcpp, digest, MASS, methods, Matrix, parallel
Depends: R (>= 3.1.0)
Suggests: mvtnorm, numDeriv, roxygen2 (>= 3.1), devtools, Rmpi, rpf (>=
        0.45), snowfall, lme4
LazyLoad: yes
LazyData: yes
Collate: '0ClassUnion.R' 'cache.R' 'MxBaseNamed.R' 'MxData.R'
        'MxDataWLS.R' 'DefinitionVars.R' 'MxReservedNames.R'
        'MxNamespace.R' 'MxSearchReplace.R' 'MxFlatSearchReplace.R'
        'MxUntitled.R' 'MxAlgebraFunctions.R' 'MxExponential.R'
        'MxMatrix.R' 'DiagMatrix.R' 'FullMatrix.R' 'IdenMatrix.R'
        'LowerMatrix.R' 'SdiagMatrix.R' 'StandMatrix.R' 'SymmMatrix.R'
        'UnitMatrix.R' 'ZeroMatrix.R' 'MxMatrixFunctions.R'
        'MxAlgebra.R' 'MxCycleDetection.R' 'MxDependencies.R'
        'MxAlgebraConvert.R' 'MxAlgebraTransform.R' 'MxSquareBracket.R'
        'MxEval.R' 'MxRename.R' 'MxPath.R' 'MxObjectiveMetaData.R'
        'MxRAMMetaData.R' 'MxExpectation.R' 'MxExpectationNormal.R'
        'MxExpectationRAM.R' 'MxExpectationLISREL.R' 'MxFitFunction.R'
        'MxFitFunctionAlgebra.R' 'MxFitFunctionML.R'
        'MxFitFunctionMultigroup.R' 'MxFitFunctionRow.R'
        'MxFitFunctionWLS.R' 'MxRAMObjective.R' 'MxLISRELObjective.R'
        'MxFIMLObjective.R' 'MxMLObjective.R' 'MxRowObjective.R'
        'MxAlgebraObjective.R' 'MxBounds.R' 'MxConstraint.R'
        'MxInterval.R' 'MxTypes.R' 'MxCompute.R' 'MxModel.R'
        'MxRAMModel.R' 'MxLISRELModel.R' 'MxModelDisplay.R'
        'MxFlatModel.R' 'MxMultiModel.R' 'MxModelFunctions.R'
        'MxModelParameters.R' 'MxUnitTesting.R' 'MxApply.R' 'MxRun.R'
        'MxRunHelperFunctions.R' 'MxSummary.R' 'MxCompare.R'
        'MxSwift.R' 'MxOptions.R' 'MxThreshold.R' 'OriginalMx.R'
        'MxGraph.R' 'MxGraphviz.R' 'MxDeparse.R' 'MxCommunication.R'
        'MxRestore.R' 'MxVersion.R' 'MxPPML.R' 'MxRAMtoML.R' 'MxDiff.R'
        'MxErrorHandling.R' 'MxDetectCores.R' 'MxSaturatedModel.R'
        'omxBrownie.R' 'omxConstrainThresholds.R' 'omxGetNPSOL.R'
        'MxFitFunctionR.R' 'MxRObjective.R'
        'MxExpectationHiddenMarkov.R' 'MxExpectationMixture.R'
        'MxExpectationStateSpace.R' 'MxExpectationBA81.R'
        'MxFitFunctionGREML.R' 'MxExpectationGREML.R' 'MxMI.R'
        'MxFactorScores.R' 'MxRobustSE.R' 'MxAvailableOptimizers.R'
        'MxTryHard.R' 'MxSE.R' 'MxAutoStart.R' 'MxRetro.R' 'zzz.R'
Version: 2.9.6
RoxygenNote: 6.0.1
NeedsCompilation: yes
Packaged: 2018-03-16 19:16:17 UTC; joshua
Author: Steven M. Boker [aut],
  Michael C. Neale [aut],
  Hermine H. Maes [aut],
  Michael J. Wilde [ctb],
  Michael Spiegel [aut],
  Timothy R. Brick [aut],
  Ryne Estabrook [aut],
  Timothy C. Bates [aut],
  Paras Mehta [ctb],
  Timo von Oertzen [ctb],
  Ross J. Gore [aut],
  Michael D. Hunter [aut],
  Daniel C. Hackett [ctb],
  Julian Karch [ctb],
  Andreas M. Brandmaier [ctb],
  Joshua N. Pritikin [aut, cre],
  Mahsa Zahery [aut],
  Robert M. Kirkpatrick [aut],
  Yang Wang [ctb],
  Charles Driver [ctb],
  Massachusetts Institute of Technology [cph],
  S. G. Johnson [cph],
  Association for Computing Machinery [cph],
  Dieter Kraft [cph],
  Stefan Wilhelm [cph],
  Manjunath B G [cph]
Maintainer: Joshua N. Pritikin <jpritikin@pobox.com>
Repository: CRAN
Date/Publication: 2018-03-18 14:05:17 UTC
Built: R 3.4.1; x86_64-w64-mingw32; 2018-08-04 13:55:00 UTC; windows
Archs: x64
